#!/bin/sh

####################
### 引数チェック ###

if [ $# -ne 1 ]; then
  echo "ERROR:引数が指定されていないか、多すぎます。"
  echo " 【使い方】./date_calculation.sh YYYY-MM-DD-hh:mm"
  echo "           YYYY-MM-DD-hh:mmにはログ分析の開始日時を指定してください。"
  exit
elif [[ ! "$1" =~ ^[0-9]{4}-[0-9]{2}-[0-9]{2}-[0-9]{2}:[0-9]{2}$ ]]; then
  echo "ERROR:指定された引数の構文が違います。"
  echo " 【使い方】./date_calculation.sh YYYY-MM-DD-hh:mm"
  echo "           YYYY-MM-DD-hh:mmにはログ分析の開始日時を指定してください。"
  exit
fi

# 指定された日時を、YYYY, MM, DD, hh, mmに分解して配列に格納
START_DATE=(`echo $1 | awk -F "-" '{print $1" "$2" "$3" "$4}'`)
START_TIME=(`echo ${START_DATE[3]} | awk -F ":" '{print $1" "$2}'`)

# 指定された日時から1秒後
ONE_SEC_AFTER_DATE=(`date --date "${START_DATE[0]}${START_DATE[1]}${START_DATE[2]} ${START_TIME[0]}:${START_TIME[1]} 1sec" "+%Y-%m-%d-%H:%M:%S"`)

# 指定された日時から2分後
TWO_MIN_AFTER_DATE=(`date --date "${START_DATE[0]}${START_DATE[1]}${START_DATE[2]} ${START_TIME[0]}:${START_TIME[1]} 2min" "+%Y-%m-%d-%H:%M:%S"`)

# 指定された日時から3時間後
THREE_HOUR_AFTER_DATE=(`date --date "${START_DATE[0]}${START_DATE[1]}${START_DATE[2]} ${START_TIME[0]}:${START_TIME[1]} 3hour" "+%Y-%m-%d-%H:%M:%S"`)

# 指定された日時から4日後
FOUR_DAY_AFTER_DATE=(`date --date "${START_DATE[0]}${START_DATE[1]}${START_DATE[2]} ${START_TIME[0]}:${START_TIME[1]} 4day" "+%Y-%m-%d-%H:%M:%S"`)

# 指定された日時から1秒前
ONE_SEC_BEFORE_DATE=(`date --date "${START_DATE[0]}${START_DATE[1]}${START_DATE[2]} ${START_TIME[0]}:${START_TIME[1]} 1sec ago" "+%Y-%m-%d-%H:%M:%S"`)

# 指定された日時から2分前
TWO_MIN_BEFORE_DATE=(`date --date "${START_DATE[0]}${START_DATE[1]}${START_DATE[2]} ${START_TIME[0]}:${START_TIME[1]} 2min ago" "+%Y-%m-%d-%H:%M:%S"`)

# 指定された日時から3時間前
THREE_HOUR_BEFORE_DATE=(`date --date "${START_DATE[0]}${START_DATE[1]}${START_DATE[2]} ${START_TIME[0]}:${START_TIME[1]} 3hour ago" "+%Y-%m-%d-%H:%M:%S"`)

# 指定された日時から4日前
FOUR_DAY_BEFORE_DATE=(`date --date "${START_DATE[0]}${START_DATE[1]}${START_DATE[2]} ${START_TIME[0]}:${START_TIME[1]} 4day ago" "+%Y-%m-%d-%H:%M:%S"`)


################
### 結果表示 ###

echo "指定した日時 -> ${START_DATE[0]}-${START_DATE[1]}-${START_DATE[2]} ${START_TIME[0]}:${START_TIME[1]}"
echo ""

echo "指定された日時から1秒後   -> ${ONE_SEC_AFTER_DATE}"
echo "指定された日時から2分後   -> ${TWO_MIN_AFTER_DATE}"
echo "指定された日時から3時間後 -> ${THREE_HOUR_AFTER_DATE}"
echo "指定された日時から4日後   -> ${FOUR_DAY_AFTER_DATE}"
echo ""

echo "指定された日時から1秒前   -> ${ONE_SEC_BEFORE_DATE}"
echo "指定された日時から2分前   -> ${TWO_MIN_BEFORE_DATE}"
echo "指定された日時から3時間前 -> ${THREE_HOUR_BEFORE_DATE}"
echo "指定された日時から4日前   -> ${FOUR_DAY_BEFORE_DATE}"


