#!/bin/sh

####################
### 引数チェック ###

if [ $# -ne 1 ]; then
  echo "ERROR:引数が指定されていないか、多すぎます。"
  echo " 【使い方】./detect_dos.sh YYYY-MM-DD-hh:mm"
  echo "           YYYY-MM-DD-hh:mmにはログ分析の開始日時を指定してください。"
  exit
elif [[ ! "$1" =~ ^[0-9]{4}-[0-9]{2}-[0-9]{2}-[0-9]{2}:[0-9]{2}$ ]]; then
  echo "ERROR:指定された引数の構文が違います。"
  echo " 【使い方】./detect_dos.sh YYYY-MM-DD-hh:mm"
  echo "           YYYY-MM-DD-hh:mmにはログ分析の開始日時を指定してください。"
  exit
fi

############################
### 各種パラメータの設定 ###

# ログ分析開始時間は引数で指定。YYYY, MM, DD, hh, mmに分解して配列に格納
START_DATE=(`echo $1 | awk -F "-" '{print $1" "$2" "$3" "$4}'`)
START_TIME=(`echo ${START_DATE[3]} | awk -F ":" '{print $1" "$2}'`)

# ログ分析の終了時間。ここでは引数で指定した開始時間から5分間を分析対象とする
END_DATE=(`date --date "${START_DATE[0]}${START_DATE[1]}${START_DATE[2]} ${START_TIME[0]}:${START_TIME[1]} 5min" "+%Y %m %d"`)
END_TIME=(`date --date "${START_DATE[0]}${START_DATE[1]}${START_DATE[2]} ${START_TIME[0]}:${START_TIME[1]} 5min" "+%H %M"`)

# while文で使用するための日時データの初期設定
START_DATE_TMP="${START_DATE[0]}${START_DATE[1]}${START_DATE[2]} ${START_TIME[0]}:${START_TIME[1]}:00"
END_DATE_TMP="${END_DATE[0]}${END_DATE[1]}${END_DATE[2]} ${END_TIME[0]}:${END_TIME[1]}:00"

# 解析対象のログ名（実際のパス、ログ名に合わせて変更すること）
LOGFILE_PATH="/var/log/httpd"
LOGFILE_NAME="access_log"

# 作業用tmpファイルの指定（TMPFILE_PATHのディレクトリが存在しない場合は作成しておくこと）
TMPFILE_PATH="/tmp/sample_log"
TMPFILE_NAME="tmp.txt"
TMPFILE_NAME2="tmp2.txt"

# 結果ファイルの指定（RESULT_FILE_PATHのディレクトリが存在しない場合は作成しておくこと）
RESULT_FILE_PATH="/tmp/result"
RESULT_FILE_NAME="result_${START_DATE[0]}${START_DATE[1]}${START_DATE[2]}${START_TIME[0]}${START_TIME[1]}.log"
cat /dev/null > ${RESULT_FILE_PATH}/${RESULT_FILE_NAME}
echo "分析開始日時：${START_DATE_TMP}" >> ${RESULT_FILE_PATH}/${RESULT_FILE_NAME}
echo "分析終了日時：`date --date "${END_DATE_TMP} 1sec ago" "+%Y%m%d %H:%M:%S"`" >> ${RESULT_FILE_PATH}/${RESULT_FILE_NAME}

# ログ分析開始/終了時間の月表示をログの出力フォーマットにあわせて変換する
START_DATE[1]=`date --date "${START_DATE[0]}${START_DATE[1]}${START_DATE[2]}" "+%b"`
END_DATE[1]=`date --date "${END_DATE[0]}${END_DATE[1]}${END_DATE[2]}" "+%b"`

# 集計する時間間隔(秒)
INTERVAL=60

# DoS攻撃とする閾値
THRESHOLD=100

# 結果メール送信用パラメータ
MAIL_ADDR="送信先のメールアドレスを記載"
MYNAME=`basename ${0}`
MYHOST=`uname -n`


#####################################
### サブルーチン:結果メール送信用 ###
SEND_MAIL()
{
  echo -e "${1} \\n" | mail -s "${MYNAME} report from ${MYHOST} ${RESULT_FILE_NAME}" ${MAIL_ADDR} &
  echo "結果をメールで送信しました"
}


################################################################
### メインルーチン:ログファイルから指定された時刻を抽出する ###

# INTERVALのインクリメントを行うwhile文
while [ "${START_DATE_TMP}" != "${END_DATE_TMP}" ]
do
  # 秒のインクリメントを行うwhile文の日時データ初期化
  START_DATE_TMP2=`date --date "${START_DATE_TMP}" "+%Y%m%d %H:%M:%S"`
  END_DATE_TMP2=`date --date "${START_DATE_TMP} ${INTERVAL}sec" "+%Y%m%d %H:%M:%S"`

  # tmpファイルの初期化
  cat /dev/null > ${TMPFILE_PATH}/${TMPFILE_NAME}
  cat /dev/null > ${TMPFILE_PATH}/${TMPFILE_NAME2}

  # 秒のインクリメントを行うwhile文
  # 対象ログから、該当する時間の部分を抽出する
  while [ "${START_DATE_TMP2}" != "${END_DATE_TMP2}" ]
  do
    # 分析対象となるログの日時フォーマットに合わせる
    SEARCH_DATE=`date --date "${START_DATE_TMP2}" "+%d/%b/%Y:%H:%M:%S"`

    # 対象ログを抽出して、一時ファイルへ出力
    fgrep ${SEARCH_DATE} ${LOGFILE_PATH}/${LOGFILE_NAME} >> ${TMPFILE_PATH}/${TMPFILE_NAME}

    # 秒をインクリメントする
    START_DATE_TMP2=`date --date "${START_DATE_TMP2} 1sec" "+%Y%m%d %H:%M:%S"`
  done

  # 生成された一時ファイルからアクセス元IPアドレス毎にアクセス数を集計
  cat ${TMPFILE_PATH}/${TMPFILE_NAME} | awk -F " " '{print $1}' | sort | uniq -c | sort -rn >> ${TMPFILE_PATH}/${TMPFILE_NAME2}

  # アクセス元IPアドレス毎のアクセス数から閾値を超えているものを結果ファイルへ出力
  echo "--- ${START_DATE_TMP}-`date --date "${START_DATE_TMP2} 1sec ago" "+%Y%m%d %H:%M:%S"`" >> ${RESULT_FILE_PATH}/${RESULT_FILE_NAME}
  RESULT_COUNT=0
  while read line; do
    CHECK_THRESHOLD=(`echo $line | awk -F " " '{print $1" "$2}'`)
    if [ ${CHECK_THRESHOLD[0]} -ge ${THRESHOLD} ]; then
      echo "${CHECK_THRESHOLD[1]} --> ${CHECK_THRESHOLD[0]} access" >> ${RESULT_FILE_PATH}/${RESULT_FILE_NAME}
      RESULT_COUNT=`expr ${RESULT_COUNT} + 1`
    fi
  done < ${TMPFILE_PATH}/${TMPFILE_NAME2}

  # 閾値越えが無かった場合のメッセージ出力
  if [ ${RESULT_COUNT} -eq 0 ]; then
    echo "閾値を超えるアクセスはありませんでした。" >> ${RESULT_FILE_PATH}/${RESULT_FILE_NAME}
  fi
  echo "" >> ${RESULT_FILE_PATH}/${RESULT_FILE_NAME}

  echo "${START_DATE_TMP}の分析が終了しました"
  START_DATE_TMP=`date --date "${START_DATE_TMP} ${INTERVAL}sec" "+%Y%m%d %H:%M:%S"`
done

# 閾値越えがあった場合にメールでアラートを出す
CHECK_RESULT_FILE=0
CHECK_RESULT_FILE=`grep "\-\->" ${RESULT_FILE_PATH}/${RESULT_FILE_NAME} | wc -l`
if [ ${CHECK_RESULT_FILE} -ne 0 ]; then
  echo "閾値を超えるアクセスが検出されました"
  SEND_MAIL "`cat ${RESULT_FILE_PATH}/${RESULT_FILE_NAME}`"
elif [ ${CHECK_RESULT_FILE} -eq 0 ]; then
  echo "閾値を超えるアクセスはありませんでした"
fi

rm -f ${TMPFILE_PATH}/${TMPFILE_NAME}
rm -f ${TMPFILE_PATH}/${TMPFILE_NAME2}
